# =====================================
# generator=datazen
# version=3.1.3
# hash=c09b8fc185aee1d63053118e6abd919e
# =====================================

"""
Useful defaults and other package metadata.
"""

DESCRIPTION = "A network-application TUI using textual."
PKG_NAME = "conntextual"
VERSION = "1.3.0"
