# =====================================
# generator=datazen
# version=3.1.3
# hash=7c7cd6fb4915a6dbda6dfbc90aaf2fb5
# =====================================
"""
conntextual - Package's default entry-point.
"""

# built-in
import sys

# internal
from conntextual.entry import main

if __name__ == "__main__":
    sys.exit(main(sys.argv))
